'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createNetworkInterface = createNetworkInterface;

var _ramda = require('ramda');

var _networkInterface = require('apollo-client/transport/networkInterface');

var _phoenix = require('phoenix');

var applyWares = function applyWares(ctx, wares) {
  return new Promise(function (resolve, reject) {
    var queue = function queue(funcs, scope) {
      var next = function next() {
        if (funcs.length > 0) {
          var f = funcs.shift();
          f && (f.applyMiddleware || f.applyAfterware).apply(scope, [ctx, next]);
        } else {
          resolve(ctx);
        }
      };
      next();
    };
    queue(wares.slice());
  });
};

var createSocket = function createSocket(options) {
  var uri = options.uri;

  var Socket = options.Socket || function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return new (Function.prototype.bind.apply(_phoenix.Socket, [null].concat(args)))();
  };
  return Socket(uri, options);
};

var getSocket = function getSocket(sockets, options) {
  if (options.logger === true) {
    options.logger = function (kind, msg, data) {
      return console.log('Apollo websocket ' + kind + ': ' + msg, data);
    };
  }

  var uri = options.uri;

  if (!uri) throw 'Missing apollo websocket options.uri';

  if (sockets[uri]) {
    return sockets[uri];
  }

  return sockets[uri] = {
    conn: createSocket(options),
    channels: {}
  };
};

var getChannel = function getChannel(socket, options) {
  var channel = options.channel;


  if (!channel) throw "Missing options.channel";
  if (!channel.topic) throw "Missing options.channel.topic";

  if (socket.channels[channel.topic]) {
    return socket.channels[channel.topic];
  } else {
    return socket.channels[channel.topic] = {
      conn: socket.conn.channel(channel.topic, channel.params || {}),
      queue: []
    };
  }
};

var socketConnect = function socketConnect(sockets, options) {
  return new Promise(function (resolve, reject) {
    var socket = getSocket(sockets, options);
    if (socket.conn.isConnected()) {
      resolve(socket);
    } else {
      socket.conn.onOpen(function (_) {
        return resolve(socket);
      });
      socket.conn.onError(reject);
      socket.conn.connect();
    }
  });
};

var channelJoin = function channelJoin(socket, options) {
  return new Promise(function (resolve, reject) {
    var channel = getChannel(socket, options);
    if (channel.conn.joinedOnce) {
      resolve(channel);
    } else if (!channel.conn.isJoining()) {
      channel.conn.join().receive('ok', function (_) {
        return resolve(channel);
      }).receive('error', reject).receive('timeout', reject);
    }
  });
};

var queryMessage = function queryMessage(_ref) {
  var channel = _ref.channel;
  return channel.queryMessage || channel.in_msg || 'gql';
};

var alwaysResolve = function alwaysResolve(promiser) {
  return function () {
    return promiser.apply(undefined, arguments).catch(function (err) {
      return err;
    });
  };
};

var performQuery = function performQuery(_ref2) {
  var channel = _ref2.channel,
      context = _ref2.context;
  return new Promise(function (resolve, reject) {
    var options = context.options,
        request = context.request;


    var message = queryMessage(options);
    var payload = (0, _networkInterface.printRequest)(request);

    channel.conn.push(message, payload).receive('ok', resolve).receive('ignore', resolve).receive('error', reject).receive('timeout', reject);
  });
};

var subscriptionEventName = function subscriptionEventName(_ref3) {
  var context = _ref3.context;
  return function (subscriptionResponse) {
    return new Promise(function (resolve, reject) {
      var subscriptionEvent = context.options.channel.subscriptionEvent;

      if (typeof subscriptionEvent !== 'function') {
        throw '\nexpected options.channel.subscriptionEvent to be a function with signature:\n\n  (subscriptionResponse, {options}) => subscriptionEventName\n';
      }
      resolve(subscriptionEvent(subscriptionResponse, context));
    });
  };
};

var subscribeToEvent = function subscribeToEvent(subMeta) {
  return function (eventName) {
    return new Promise(function (resolve, reject) {
      var channel = subMeta.channel,
          context = subMeta.context,
          subCallback = subMeta.subCallback,
          responseMiddleware = subMeta.responseMiddleware;

      var handler = function handler(eventData) {
        (0, _ramda.pipeP)(responseMiddleware, function (responseContext) {
          var response = responseContext.response;

          subCallback( /*error*/null, response);
          return responseContext;
        })(eventData);
      };
      channel.conn.on(eventName, handler);
      var subscription = {
        // TODO: send unsubscribe message to server
        unsubscribe: function unsubscribe(_) {
          return channel.conn.off(eventName, handler);
        }
      };
      resolve(subscription);
    });
  };
};

var performSubscribe = function performSubscribe(_ref4) {
  var subCallback = _ref4.subCallback,
      responseMiddleware = _ref4.responseMiddleware;
  return function (_ref5) {
    var channel = _ref5.channel,
        context = _ref5.context;

    var subMeta = { channel: channel, context: context, subCallback: subCallback, responseMiddleware: responseMiddleware };
    return (0, _ramda.pipeP)(performQuery, subscriptionEventName(subMeta), subscribeToEvent(subMeta))({ channel: channel, context: context });
  };
};

var senderChannel = function senderChannel(sockets) {
  return function (context) {
    var options = context.options;

    return (0, _ramda.pipeP)(function (_) {
      return socketConnect(sockets, options);
    }, function (socket) {
      return channelJoin(socket, options);
    }, function (channel) {
      return { channel: channel, context: context };
    })();
  };
};

var isEmptyOrNil = (0, _ramda.either)(_ramda.isEmpty, _ramda.isNil);

var responseData = function responseData(_ref6) {
  var response = _ref6.response;

  return new Promise(function (resolve, reject) {
    if (isEmptyOrNil(response)) {
      reject('No response');
    } else if (!isEmptyOrNil(response.data)) {
      resolve(response);
    } else {
      reject(response);
    }
  });
};

var newSubscriptionId = function newSubscriptionId(_) {
  return (Date.now().toString(36) + Math.random().toString(36).substr(2, 8)).toUpperCase();
};

function createNetworkInterface(ifaceOpts) {
  var sockets = {};
  var middlewares = [];
  var afterwares = [];

  var use = (0, _ramda.map)(function (item) {
    return middlewares.push(item);
  });
  var useAfter = (0, _ramda.map)(function (item) {
    return afterwares.push(item);
  });

  var requestMiddleware = function requestMiddleware(request) {
    var options = (0, _ramda.clone)(ifaceOpts);
    return applyWares({ request: request, options: options }, middlewares);
  };

  var responseMiddleware = function responseMiddleware(response) {
    var options = (0, _ramda.clone)(ifaceOpts);
    return applyWares({ response: response, options: options }, afterwares);
  };

  var query = (0, _ramda.pipeP)(requestMiddleware, senderChannel(sockets), alwaysResolve(performQuery), responseMiddleware, responseData);

  var subscribe = function subscribe(request, subCallback) {
    return (0, _ramda.pipeP)(requestMiddleware, senderChannel(sockets), performSubscribe({ subCallback: subCallback, responseMiddleware: responseMiddleware }))(request);
  };

  return { query: query, use: use, useAfter: useAfter, subscribe: subscribe };
}